%post
#!/bin/bash
# Shebang will be replaced with "%post"
# Using shebang here to easify checking in shellcheck

set -e #fail on any error
set -u #fail on not defined variables
#set -f #disable globbing

# Some scripts may need to detect if they are executed by Anaconda or not
export I_AM_ANACONDA=1

# Force the log of this script to be in English to guarantee that
# developers of this script will be able to understand it
# https://pubs.opengroup.org/onlinepubs/9699919799/basedefs/V1_chap08.html
# https://bugzilla.altlinux.org/show_bug.cgi?id=37231
export LC_ALL="C.UTF-8"

echo_err(){
	echo "$@" 1>&2
}

_cleanup_pkgs(){
        local virt
        virt="$(systemd-detect-virt)"
        # When this script is run in chroot, /dev and /sys are bind-mounted from host
        local vbox_pkgs=""
        if [ "$virt" != oracle ]; then
                vbox_pkgs="xorg-x11-drv-vbox  virtualbox-guest-additions VirtualBox-kmod"
        fi
	local vmware_pkgs=""
        if [ "$virt" != vmware ]; then
                vmware_pkgs="open-vm-tools"
        fi
	local virtio_pkgs=""
        # from /lib/udev/rules.d/70-spice-vdagentd.rules
        if [ ! -e /dev/virtio-ports/com.redhat.spice.0 ]; then
                virtio_pkgs="spice-vdagent"
        fi
        if [ ! -e /dev/virtio-ports/ovirt-guest-agent.0 ] or [ ! -e /dev/virtio-ports/com.redhat.rhevm.vdsm ] ; then
                virtio_pkgs="${virtio_pkgs} qemu-guest-agent"
        fi

	if [ ! -e /dev/virtio-ports/org.qemu.guest_agent.0 ]; then
                virtio_pkgs="${virtio_pkgs} qemu-guest-agent"
        fi
	local bootloaders_pkgs=""
        if [ ! -e /sys/firmware/efi ]; then
                bootloaders_pkgs="grub2-efi"
        fi

        # shellcheck disable=SC2046,SC2086
        dnf remove -y --setopt=metadata_expire=-1 $(rpm -qa --qf '%{NAME} ' ${vbox_pkgs} ${vmware_pkgs} ${virtio_pkgs} ${bootloaders_pkgs})
}


_import_rpm_gpg_keys(){
        if ! rpm -q redos-release >/dev/null 2>&1; then
                echo_err "redos-repos-keys is not installed, skipping import of GPG keys"
                return 0
        fi
        rpm -ql redos-release | grep '^/etc/pki/rpm-gpg/' |
        while read -r line
        do
                if [ -f "$line" ]; then
                        rpm --import "$line"
                fi
        done
}

_dnf_autoremove(){
        dnf autoremove -y --setopt=metadata_expire=-1 --exclude gnome-kiosk
}

_get_lang(){
        # $LANG is not the same as the chosen language in Anaconda,
        # so sourcing it, but using previously set $LANG if $LANG is not defined
        # in /etc/locale.conf for any reason (that must never happen, Anaconda create this file)
        if [ -r /etc/locale.conf ]; then
                . /etc/locale.conf
        fi
        # for set -u
        LANG="${LANG:-}"
        if [ -z "$LANG" ]; then
                # stdout of this function is captured into a value of a variable which is
                # then checked for being not empty, it's important to not confuse stdout and stderr
                echo_err '$LANG is not set, skipping installation of packages with localization'
                return 1
        fi
        local lang
        # ru_RU.UTF-8 -> ru, gu.UTF-8 -> gu (https://docs.moodle.org/dev/Table_of_locales)
        lang="$(echo "$LANG" | awk -F '.' '{print $1}' | awk -F '_' '{print $1}')"
        # TODO: improve regexp
        if [ -z "$lang" ] || ! [[ "$lang" =~ [a-z] ]]; then
                echo_err 'Error converting $LANG to a short name'
                return 1
        fi
        echo "$lang"
}

_cleanup_pkgs(){
        local virt
        virt="$(systemd-detect-virt)"
        # When this script is run in chroot, /dev and /sys are bind-mounted from host
        local vbox_pkgs=""
        if [ "$virt" != oracle ]; then
                vbox_pkgs="xorg-x11-drv-vbox  virtualbox-guest-additions VirtualBox-kmod"
        fi
	local vmware_pkgs=""
        if [ "$virt" != vmware ]; then
                vmware_pkgs="open-vm-tools"
        fi
	local virtio_pkgs=""
        # from /lib/udev/rules.d/70-spice-vdagentd.rules
        if [ ! -e /dev/virtio-ports/com.redhat.spice.0 ]; then
                virtio_pkgs="spice-vdagent"
        fi
	if [ ! -e /dev/virtio-ports/org.qemu.guest_agent.0 ]; then
                virtio_pkgs="${virtio_pkgs} qemu-guest-agent"
        fi
	local bootloaders_pkgs=""
        if [ ! -e /sys/firmware/efi ]; then
                bootloaders_pkgs="grub2-efi"
        fi
	local anaconda_pkgs
        anaconda_pkgs="$(rpm -qa --qf '%{NAME} ' anaconda*)"
        # shellcheck disable=SC2046,SC2086
        dnf remove -y --setopt=metadata_expire=-1 $(rpm -qa --qf '%{NAME} ' ${vbox_pkgs} ${vmware_pkgs} ${virtio_pkgs} ${bootloaders_pkgs})
}


_refresh_dnf_and_install_locales(){
        # Certified distros must be fully functional without internet connection,
        # avoid accident mistakes if they are first tested with internet
        # dnf may freeze for quite a long time if there is no network
        if ! {  timeout 10 sh -c 'echo > /dev/tcp/mirror.yandex.ru/80' ;}; then
                echo_err "Network seems to be not reachable, skipping dnf operations"
                return 1
        fi
        dnf refresh
        # Sourcing /etc/locale.conf is a subshell to avoid leakage of those variables into the current environment
        # to avoid running dnf etc. in another language what will make the log of this script not understandable
        local lang
        lang="$(_get_lang)"
        [ -n "$lang" ]
        local package_to_install
        package_to_install="glibc-langpack-${lang}"
        if ! rpm -q "$package_to_install" >/dev/null 2>&1; then
                dnf install -y "$package_to_install"
        fi
}

_dnf_autoremove(){
        dnf autoremove -y --setopt=metadata_expire=-1
}

_is_in_domain(){
        if ! test -f /etc/krb5.conf; then
                return 1
        fi
        if rpm -Vf /etc/krb5.conf | grep -q ' /etc/krb5.conf$'; then
                return 0
        fi
        for i in /etc/krb5.conf.d/*
        do
                if [ "$i" != "/etc/krb5.conf.d/*" ] && ! rpm -qf "$i" >/dev/null 2>&1; then
                        return 0
                fi
        done
        return 1
}

# Allow to put any scripts to /etc/anaconda-scripts.d/post-install/ in ISO
_run_additional_scripts(){
        systemctl disable firewalld.service
	if [ -d /etc/anaconda-scripts.d/post-install ]; then
		pushd /etc/anaconda-scripts.d/post-install
		for i in *
		do
			"./$i" || :
		done
		popd
		# /etc/anaconda-scripts.d/post-install belongs to an RPM package and will be removed on its uninstall
		for i in /etc/anaconda-scripts.d/post-install/*
		do
			if ! rpm -qf "$i" ; then
				rm -fvr "$i"
			fi
		done
	fi
}

# XXX Anaconda runs update-grub2 before creating /etc/locale.conf
# This leads to incorrect language of Grub
# TODO: fix Anaconda properly
_update_grub(){
	# $LANG has prority over $LC_ALL.
	# $LANG is set in /etc/locale.conf, $LC_ALL is set,
	# but $LC_ALL is set in the beginning of this script.
	# () to avoid leakage of variables.
        if [ -e /sys/firmware/efi ]; then
                ( . /etc/locale.conf && LC_ALL="$LANG" grub2-mkconfig -o /boot/efi/EFI/redos/grub.cfg )
        else
                ( . /etc/locale.conf && LC_ALL="$LANG" grub2-mkconfig -o /boot/grub2/grub.cfg )
        fi
}

_main(){
        _cleanup_pkgs ||:
        _import_rpm_gpg_keys || :
        _dnf_autoremove || :
	_run_additional_scripts || :
	_update_grub || :
}

# ! This script will be deleted by dnf after execution,
# so the _whole_ script must be loaded into memory
# https://unix.stackexchange.com/questions/331837/
# https://habr.com/ru/post/500832/
mkdir -p /var/log/anaconda
{ set -x; _main 2>&1 ;} | tee /var/log/anaconda/redos-postinstall.log
sync
%end
